#version 430

struct Material {
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
    float shininess;
};

struct Light {
    vec3 position;
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
};

uniform Material material;
uniform Light directionalLight;

uniform sampler2D shadowMap;

uniform bool threshold = false;
uniform vec3 lightPos;
uniform vec3 viewPos;
uniform float alpha = 1;

in vec3 fragNormal;
in vec2 fragUV;
in vec3 fragPos;
in vec3 fragViewPos;
in vec4 fragPosLightSpace;

out vec4 outputColor;

float calculateShadow(vec4 fragPosLightSpace) {
	vec3 projCoords = fragPosLightSpace.xyz;
    projCoords = projCoords * 0.5 + 0.5;
    float closestDepth = texture(shadowMap, projCoords.xy).r; 
    float currentDepth = projCoords.z;
	// set bias to avoid shadow acne
	float bias = 0.001;

	float shadow = 0.0;
	vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
	for(int x = -1; x <= 1; ++x) {
		for(int y = -1; y <= 1; ++y) {
			float pcfDepth = texture(shadowMap, projCoords.xy + vec2(x, y) * texelSize).r;
			shadow += currentDepth - bias > pcfDepth ? 1.0 : 0.0;
		}
	}

    return shadow / 9.0;
}

void main() {

	vec3 normalNorm = normalize(fragNormal);
	vec3 viewDir = normalize(fragViewPos - fragPos);

	vec3 dirLightDir = vec3(0) - directionalLight.position;
	vec3 dirLightDirNorm = normalize(dirLightDir);
	vec3 dirLightRef = reflect(dirLightDirNorm, normalNorm);

	vec3 dirLightAmbient = directionalLight.ambient * material.diffuse;
	vec3 dirLightDiffuse = directionalLight.diffuse * max(dot(normalNorm, -dirLightDirNorm), 0) * material.diffuse;
	vec3 dirLightSpecular = directionalLight.specular  * pow(max(dot(viewDir, dirLightRef), 0), material.shininess) * material.shininess;

	vec3 ambient = dirLightAmbient;
	vec3 diffuse = dirLightDiffuse;
	vec3 specular = dirLightSpecular;

	float shadow = calculateShadow(fragPosLightSpace);

	vec3 result = ambient + (1 - shadow) * (diffuse + specular);

	if (threshold) {
		float brightness = dot(result, vec3(0.95, 0.85, 0.1));
		if (brightness > 1.0)
			outputColor = vec4(result, alpha);
		else
			outputColor = vec4(0.0, 0.0, 0.0, 1.0);
	} else {
		outputColor = vec4(result, alpha);
	}

}
